
;*======================================================================================*
;*			Database structure															*
;*--------------------------------------------------------------------------------------*
;*			There are 3 sub-databases:													*
;*			1.	Word database - this database contains all the strings, from group		*
;*				name to remark. They are saved in Pascal style format sequentially		*
;*				indexed. All reference to them uses their index. The index is zero		*
;*				based. Database name: Word.dat											*
;*			2.	Password database - this database contains a record of the password		*
;*				information. All the fields which are strings refer to the strings		*
;*				in the word database. Each field has an index, from the group name		*
;*				to the remark. All indexes are zero based. Unused fields are denoted	*
;*				with -1 (0FFFFFFFFh). Database name: PW.dat								*
;*			3.	Group database - list of all the group name that have been defined		*
;*				The database contains the group name indexes. Database name: GP.dat		*
;*																						*
;*			All three databases are stored separately.									*
;*--------------------------------------------------------------------------------------*
;*			Password database															*
;*			-----------------															*
;*			Offset	Description				Byte count									*
;*			======	===========				==========									*
;*			 0 -3	Group name index			4										*
;*			 4 -7	Website name				4										*
;*			 8 -11	URL							4										* 
;*			 12-15	User name					4										* 
;*			 16-19	Password					4										* 
;*			 20-24	Remark						4										* 
;*																						*
;*			When more than one password is inserted into the same groupname, the		*
;*			database will insert the new password sequentially and in ascending			*
;*			of the website name.														*
;*																						*
;*			The database will manage the insertion and deletion of each record.			*
;*======================================================================================*

;*==================================================================================*
;*			Routine to compare two strings case sensitive null string				*
;*----------------------------------------------------------------------------------*
;*			Input:	ESI -> String 1													*
;*					EDI -> String 2													*
;*					DX =  0 case sensitive, 1 = case insensitive					*					
;*																					*
;*			Output:	EAX = 0	Same													*
;*						= 1 Different												*
;*						= 2 Higher													*
;*					EDX = Record index in database									*
;*																					*
;*			Registers preserved:	ESI, EDI										*
;*			Registers corrupted:	EAX												*
;*==================================================================================*
cmp_ASCIIz	proc
			push	esi						; save ESI
			push	edi						;  & EDI
			
			;*====	Setup environment for compare	====
			sub		edx,edx					; reset EDX
			
			;*====	Setup environment for compare	====
		cmplp:
			mov		al,[esi]				; get character
			cmp		al,0					; null character?
			je		cmpsame					; Yes, then string 
			
			mov		ah,[edi]				; get string 2 character
			cmp		al,0					; null character?
			je		cmpdiff					; Yes, then exit
			
			cmp		dx,0					; case sensitive?
			je		cmpcase					; Yes, do case sensitive
			
			and		ax,5F5Fh				; make it case insensitive

			;*====	Compare characters	====
		cmpcase:
			cmp		al,ah					; same?
			jne		cmpdiff					; No, different
			
			;*====	Go to next character	====
			inc		esi						; increment pointers
			inc		edi						;
			inc		edx						;  & index
			jmp		cmplp					;
			
			;*====	Restore stack	====
		cmpex:
			pop		edi						; restore EDI
			pop		esi						;  & ESI
			ret

			;*====	Strings are different	====
		cmpdiff:
			cmp		al,ah					; same?
			ja		cmphgher				; 

			mov		eax,1					; return different status
			jmp		cmpex					; Exit
			
			;*====	String 1 higher	====
		cmphgher:
			mov		eax,2					; return higher value status
			jmp		cmpex					; Exit

			;*====	Check if 2nd string also EOS	====
		cmpsame:
			mov		ah,[edi]				; get string 2 character
			cmp		al,0					; null character?
			jne		cmpdiff					; No, then string is different
			
			sub		eax,eax					; string is the same
			jmp		cmpex					; exit
cmp_ASCIIz 	endp

;*==================================================================================*
;*			Routine to get the string length of a string (ASCIIz string)			*
;*----------------------------------------------------------------------------------*
;*			This routine will determine the length of the ASCIIz string.			*
;*																					*
;*			Input:  ESI -> Source buffer											*
;*																					*
;*			Output:	ECX = String length	(one based)									*
;*==================================================================================*
get_strlen		proc
				push	esi						; save ptr
				sub		ecx,ecx					; reset ECX
		getstrlp:
				cmp		byte ptr [esi],0		; ASCIIz character?
				je		getstrex				; Yes, then exit
				
				inc		ecx						; increment length
				inc		esi						;  & ptr
				jmp		getstrlp				; continue till ASCIIz character is found

		getstrex:
				pop		esi						;
				ret
get_strlen		endp

;*==================================================================================*
;*			Routine to copy ASCIIz string from source to destination				*
;*----------------------------------------------------------------------------------*
;*			Routine to copy the source string into the destination and 				*
;*			register are not preserved												*
;*																					*
;*			Input:	ESI -> Source string											*
;*					EDI -> Destination buffer										*
;*																					*
;*			Output:	ECX = String length copied										*
;*==================================================================================*
copy_null	proc
			mov		al,[esi]					; get character
			cmp		al,00						; null character
			je		copyNex						; Yes, then exit
			
			mov		[edi],al					; save character in destination  buffer
			inc		ecx							; increment ECX
			inc		esi							; increment pointers
			inc		edi
			jmp		copy_null					; copy  				
	
			;*====	Restore pointers	====
		copyNex:
			ret

copy_null 	endp

;*==================================================================================*
;*			Routine to copy ASCIIz string from source to destination				*
;*----------------------------------------------------------------------------------*
;*			Input:	ESI -> Source string											*
;*					EDI -> Destination buffer										*
;*					ECX = Start destination string length							*
;*																					*
;*			Output:	ECX = String length copied										*
;*==================================================================================*
copy_ASCIIz	proc
			push	esi							; save pointers
			push	edi							;
			
			;*====	Copy string	====
		copylp:
			mov		ah,al						; copy previous character in AL
			mov		al,[esi]					; get character
			cmp		al,00						; null character
			je		copyex						; Yes, then exit
			
			mov		[edi],al					; save character in destination  buffer
			inc		ecx							; increment ECX
			inc		esi							; increment pointers
			inc		edi
			jmp		copylp						; copy  				
	
			;*====	Restore pointers	====
		copyex:
			mov		byte ptr [edi],0			; terminate 
			pop		edi
			pop		esi
			ret
copy_ASCIIz endp

;*==================================================================================*
;*			Routine to concatenate two strings										*
;*----------------------------------------------------------------------------------*
;*			This routine will save the database 							  		*
;*																					*
;*			Input:  EDI -> Main string												*
;*					ESI -> String 2 to write into main string 						*
;*																					*
;*			Output:	EAX = 	0 OK													*
;*							-1 Error												*
;*==================================================================================*
concatASCIIz proc
			;*====	Move pointer to end of ASCIIz string	====
		movelp:
			cmp		byte ptr [edi],00				; check if end of string
			je		endStr							; Yes, done
			
			inc		edi								; increment pointer
			jmp		movelp							; continue
			
			;*====	End of string reached. Copy the rest	====
		endStr:		
			call	copy_ASCIIz						; copy the second string into main string
			ret

concatASCIIz endp	


;*==================================================================================*
;*			Routine to concatenate a file name into file path						*
;*----------------------------------------------------------------------------------*
;*			This routine will save the database 							  		*
;*																					*
;*			Input:  EDI -> Main buffer												*
;*					ESI -> File name				 								*
;*																					*
;*			Output:	ECX = Main string length used									*
;*																					*
;*			Registers corrupted: EDI, ESI											*
;*==================================================================================*
concat_File	proc
			push	edi								; save the main buffer
			
			sub		ecx,ecx							; reset ECX
			sub		eax,eax							; reset EAX

			;*====	Move pointer to end of ASCIIz string	====
		movelp:
			cmp		byte ptr [edi],00				; check if end of string
			je		cendStr							; Yes, done

			mov		al,[edi]						; get the character before EOS			

			inc		ecx								; increment count
			inc		edi								; increment pointer
			jmp		movelp							; continue
			
			;*====	End of string reached. Copy the rest	====
		cendStr:
			cmp		al,5Ch							; last character a path delimiter?
			je		delimt							; Yes, then ignore adding one
					
			mov		byte ptr [edi],5Ch				; include the 
			inc		ecx								; increment length
			inc		edi								; increment ptr
		delimt:
			call	copy_ASCIIz						; copy the file name
			
			pop		edi								; restore the pointer	
			ret

concat_File endp

;*==================================================================================*
;*			Routine to concatenate path, file name into file path					*
;*----------------------------------------------------------------------------------*
;*			This routine will save the database 							  		*
;*																					*
;*			Input:  EDI -> Main string												*
;*					ESI -> Path name				 								*
;*					EBX -> File name												* 					
;*																					*
;*			Output:	ECX = Main string length used									*
;*																					*
;*			Registers corrupted: EDI, ESI											*
;*==================================================================================*
combine_path_file proc
			sub		ecx,ecx						; reset start length
			call	copy_ASCIIz					; copy into path into working path

			mov		esi,ebx						; system filename
			call	concat_File					; concentanate file name into path

			ret

combine_path_file endp

;*==================================================================================*
;*			Routine to setup all the variables for database operation				*
;*==================================================================================*
init_db		proc
			mov		[iPWCount],0				; no password count
			mov		[iGpCount],0				; no group record count
			mov		[iWdCount],0				; no word count
			mov		[icWd],-1					; no define word record index
			mov		[inWd],-1					; no new define word record index
			mov		[isWd],-1					; no search define word record index
			mov		[iRemMem],MAX_WD_MEM		; total word string memory size
			mov		[pNxtWd],offset WDDB		; point to start of database
			ret
init_db		endp

;*==================================================================================*
;*			Routine to fetch the record from the database into memory buffer		*
;*----------------------------------------------------------------------------------*
;*			This routine will save all the website information and save it  		*
;*			into the database.														*
;*																					*
;*			Input:  WCurIndex variables (zero based)								*
;*																					*
;*			Output:	Website information will be updated								*
;*==================================================================================*
GetPWRecord	proc
GetPWRecord endp

;*==================================================================================*
;*			Routine to display current website record in Tab 2 controls				*
;*==================================================================================*
show_Rec	proc	hWin:DWORD
			mov		eax,[icurGp]					; get current selected group
			call	getword_str						; EDI -> string
			invoke	SetDlgItemText,hTabDlg,IDC_EDT_GPNAME,esi		; show group name selected
			
			invoke	SetDlgItemText,hTabDlg,IDC_EDT_WebName,offset sWebName	; web name 
			invoke	SetDlgItemText,hTabDlg,IDC_EDT_URL,offset sURL			; URL
			invoke	SetDlgItemText,hTabDlg,IDC_EDT_UserName,offset sUserName	
			invoke	SetDlgItemText,hTabDlg,IDC_EDT_Password,offset sPassWord
			invoke	SetDlgItemText,hTabDlg,IDC_EDT_Remark,offset sRemark
			ret
show_Rec endp

;*==================================================================================*
;*			Routine to index to the required string in the word database			*
;*----------------------------------------------------------------------------------*
;*			Input:	EAX = Index number, number is one base							*
;*			Output:	EDI -> String													*
;*==================================================================================*
getword_str	proc
			mov		edi,offset WDDB					; EDI -> word database
getword_str	endp

;*==================================================================================*
;*			Routine to index through database to the required word record			*
;*----------------------------------------------------------------------------------*
;*			Input:	EAX = Index number, number is one base							*
;*					EDI -> Word database		 									*
;*																					*
;*			Output:	EDI -> Indexed word												*
;*==================================================================================*
indx_word			proc
					sub		edx,edx					; reset offset
					mov		bl,[edi]				; EDI = word length
					inc		edx						; increment length to adjust for word length
					add		edi,edx					; go to next word
					dec		ecx						; decrement index count
					jne		indx_word				; 
					ret
indx_word			endp

;*==================================================================================*
;*			Routine to search for a field's index in other password record			*
;*----------------------------------------------------------------------------------*
;*			Input:	EAX = Field reference index number (zero based)					*
;*					EDX = Offset to field in record									*
;*																					*
;*			Output: EDX = 0 Not found, > 0 = Found									*
;*==================================================================================*
chk_pwfld	proc
			push	ebp								; save EBP
			
			mov		ebp,eax							; save record index 
			
			;*====	Setup environment first	=====
			mov		[iindxCnt],0					; reset any index count
			
			mov		ecx,[iPWCount]					; password count
			mov		esi,offset PWDB					; ESI -> start of database 
			mov		edi,esi							; copy into EDI
			
			mov		ebx,offset iindxList			; buffer to save index list
			
			;*====	Point to the password record indexes	====
			shl		ecx,3							; X 8
			mov		eax,ecx							; copy 
			shl		ecx,1							; x 16
			add		ecx,eax							; X 24
			add		esi,ecx							; ESI -> start of record
			add		esi,edx							; ESI -> set to the expected field
			sub		edx,edx							; 1st index number
			
			;*====	Scan all the indexes loop	===================================
		chkpwrlp:
			push	esi								; save check record ptr
			
			cmp		ebp,edx							; at the same record index?
			je		chkpwrnxt						; Yes, avoid checking same record

			;*====	Check indexes in the field	==================================	
		chkpwr:
			mov		eax,[esi]						; record index #
			cmp		eax,[edi]						; same?
			je		chkpwrsame						; Yes, then return

			;*====	Go to next record	====
		chkpwrnxt:
			add		edi,PWBSize						; add the byte size
			inc		edx								; increment running record index number
			pop		esi								; restore check record ptr
			
			cmp		edx,[iPWCount]					; reached the end of database
			jne		chkpwrlp						; continue till all is checked
			
			sub		edx,edx							; return - Not found, status

		chkpwrex:
			pop		eax
			pop		ebp								; restore EBP
			
			;*====	Same index found in another record	====
		chkpwrsame:
			mov		[ebx],edx						; save the index into buffer
			inc		[iindxCnt]						;  & increment count
			pop		edi								; restore the start address to record
			jmp		chkpwrnxt						; exit
			
chk_pwfld	endp

;*==================================================================================*
;*			Routine to search for repeated indexes in other password record			*
;*----------------------------------------------------------------------------------*
;*			Input:	EAX = Password record index number (zero based)					*
;*																					*
;*			Output: EDX = 0 Not found, > 0 = Found									*
;*==================================================================================*
chk_pwindx	proc
			push	ebp								; save EBP
			push	eax
			
			mov		ebp,eax							; save record index 
			
			mov		ecx,[iPWCount]					; password count
			mov		esi,offset PWDB					; ESI -> start of database 
			mov		edi,esi							; copy into EDI
			
			;*====	Point to the password record indexes	====
			shl		ecx,3							; X 8
			mov		eax,ecx							; copy 
			shl		ecx,1							; x 16
			add		ecx,eax							; X 24
			add		esi,ecx							; ESI -> start of record
			sub		edx,edx							; 1st index number
			
			;*====	Scan all the indexes loop	===================================
		chkpwlp:
			push	esi								; save check record ptr
			mov		ecx,5							; check for 5 fields
			
			cmp		ebp,edx							; at the same record index?
			je		chkpwnxt						; Yes, avoid checking same record

			;*====	Check indexes in the field	==================================			
		chkpw:
			mov		eax,[esi]						; record index #
			cmp		eax,[edi]						; same?
			je		chkpwsame						; Yes, then return
			
			;*====	Go to next record	====
		chkpwnxt:
			add		esi,4							; adjust pointer to next record
			add		edi,4							; adjust pointer to next record
			dec		ecx								; decrement field count
			jne		chkpw							; 

			inc		edx								; increment running record index number
			pop		esi								; restore check record ptr
			
			cmp		edx,[iPWCount]					; reached the end of database
			jne		chkpwlp							; continue till all is checked
			
			sub		edx,edx							; return - Not found, status

		chkpwex:
			pop		eax
			pop		ebp								; restore EBP
			
			;*====	Same index found in another record	====
		chkpwsame:
			jmp		chkpwex							; exit
			
chk_pwindx	endp

;*==================================================================================*
;*			Routine to setup the pointers to group name database and insert			*
;*			information into the database											*
;*----------------------------------------------------------------------------------*
;*			Output:	EAX = 0 Not inserted											*
;*						= 1 Inserted												*
;*						= -1 Insufficient memory									*
;*==================================================================================*
set_gp_ins	proc
			lea		eax,sNewGroup					; ESI -> point to group name
			mov		[pStrWd],eax					; save it
			lea		eax,iGpCount					; groupname record count
			mov		[pCount],eax					;
			lea		eax,icurGp						; EAX -> point to current record index
			mov		[pcIndx],eax					;
			lea		eax,inewGp						; EAX -> point to new group index 
			mov		[pnIndx],eax
			lea		eax,isrchGp						; EAX -> point to search record index
			mov		[psIndx],eax
			lea		eax,GpDB						; EAX -> point to start of group database
			mov		[psDB],eax						;
			mov		[bWdType],1						; group name type
			lea		eax,WDDB						; EAX -> word database
			mov		[pcDB],eax						;
			sub		edx,edx							; No offset to any field
			jmp		Add_pword						; continue to insert record into database
set_gp_ins	endp

;*==================================================================================*
;*			Routine to setup the pointers to group name database and insert			*
;*			information into the database											*
;*----------------------------------------------------------------------------------*
;*			Input:	EDX = Offset to field in record									*
;*																					*
;*			Output:	EAX = 0 Not inserted											*
;*						= 1 Inserted												*
;*						= -1 Insufficient memory									*
;*==================================================================================*
set_pw_ins	proc
			lea		eax,sWebName					; ESI -> point to password
			mov		[pStrWd],eax					; save it
			lea		eax,iPWCount					; password record count
			mov		[pCount],eax					;
			lea		eax,icurPw						; EAX -> current record index
			mov		[pcIndx],eax					;
			lea		eax,inewPw
			mov		[pnIndx],eax
			lea		eax,isrchPw						; EAX -> search record index
			mov		[psIndx],eax
			lea		eax,PWDB						; EAX -> start of password database
			mov		[psDB],eax						;
			lea		eax,WDDB						; EAX -> word database
			mov		[pcDB],eax						;
			mov		[bWdType],2						; password type
			jmp		Add_pword						; continue to insert record into database
set_pw_ins	endp

;*==================================================================================*
;*			Routine to add group name or password information into database			*
;*----------------------------------------------------------------------------------*
;*			This routine assumes that all the data is in their respective			*
;*			memory location. It will insert the data into the word database			*
;*			It will update the group name or password start index into their		*
;*			respective index register. The new record of word indexes will 			*
;*			be inserted into into the respective database.							*
;*																					*
;*			Input variables															*
;*			---------------															*
;*			pStrWd				pointer to start of data to be inserted into DB		*
;*			pCount				pointer to record count								*
;*			pcIndx				pointer to current index							*
;*			pnIndx				pointer to new index								*
;*			psIndx				pointer to search index								*
;*			pcDB				pointer to start of database						*
;*			psDB				pointer to source database (group/password)			*
;*			iCpyCnt				number of string to copy							*
;*																					*
;*			Output:	EAX = 0 Not inserted											*
;*						= 1 Inserted												*
;*						= -1 Insufficient memory									*
;*==================================================================================*
Add_pword	proc
			mov		[isrchPw],-1					; indicate no password found
			mov		[inewPw],-1						;  & new password

			mov		ebx,[pnIndx]					; EBX = pointer to new record index register
			mov		dword ptr [ebx],-1				; default to none
			
			;*====	Setup first environment to check for duplicate string	====
			mov		edi,[pcDB]						; EDI -> start of database
			mov		esi,[pStrWd]					; ESI -> String
			cmp		[bWdType],2						; if to insert password info
			jne		addpwdon						; No, then continue

			add		esi,32							; ESI -> point to URL string

			;*====	Check if any records in database	====
		addpwdon:
			mov		ebx,[pCount]					; EBX -> record count
			mov		ecx,[ebx]						; ECX = record count
			cmp		ecx,0							; check if there are any records in the database
			je		addpwdins						; straight to insert

			;*----------------------------------------------*
			;*		Do a string search in the database		*
			;*----------------------------------------------*
			mov		[iCpyCnt],0						; reset field counter
			
		srchpwdlp:
			push	ecx								; save word count in database

			;&====	Check which type of database: group or password	====
			.if [bWdType] == 2
				;*====	Password database	====
				mov		eax,[edi+8]					; EAX = URL string index
			.else
				mov		eax,[edi]					; get word index
			.endif
			
			;*====	Index to the required word in the word database	====
			call	indx_word						; index to the word location
			
			;*----	Compare the strings	---------------------
			sub		ecx,ecx							;
			mov		cl,[edi]						; get string length
			inc		edi								; skip the length
			mov		edx,1							; word insenstitive
			
			call	cmp_ASCIIz						; compare the two strings
			
			cmp		eax,0							; same?
			jne		addpwdnxt						; go to next word

			;*====	Return the index to calling routine	====
			mov		[isrchPw],edx					; save the index
			sub		edx,edx							; return no insert status

			;*====	Exit point	===========
			pop		ecx								; restore stack
		addpwdex:
			pop		ax								;
			mov		edx,eax
			ret										; exit

			;*====	Go to next word record in word database	====			
		addpwdnxt:
			pop		ecx								; restore database word count
			
			sub		eax,eax							; reset EAX
			mov		al,[edi]						; AL = string length
			inc		al
			add		edi,eax							; go to next word
			dec		ecx								; decrement word count
			jne		srchpwdlp						; save 
			
			;*====	Insert the word into the database	====
			;*====	Calculate memory required for insertion	====
		addpwdins:
			mov		esi,[pStrWd]					; ESI -> start pointer to data field

			.if [bWdType] == 2
				;*====	Password record	===============
				inc		[iCpyCnt]					; increment number of fields
				add		esi,32						; go to next field
				
				.while [iCpyCnt] < 6
					call	get_strlen				; get string length
					inc		ecx						; include string length byte
					add		edx,ecx					; EDX = total memory required
					
					.if [iCpyCnt] != 3
						;*====	At user name field	====
						add		esi,32				; go to password field
					.elseif 
						add		esi,255				; go to other fields
					.endif
					
					inc		[iCpyCnt]				; increment number of fields
				.endw
			.else
				;*====	Process for group name	====
				sub		edx,edx						; reset total memory required
				call	get_strlen					; get string length
;				add		edx,ecx						; EDX = total memory required
				inc		ecx							; include the string length
				mov		edx,ecx						; include string length
			.endif

			;*====	Check for remaining memory space	====
			mov		eax,[iRemMem]					; EAX = remaining memory
			cmp		ecx,eax							; enough memory to insert string?
			jb		addpwdmem						; Yes, sufficient memory
			
			;*====	Insufficient memory	====
			mov		edx,-1							; return insufficient memory
			jmp		addpwdex							;
		
			;*====	Sufficient memory to insert word(s)	====	
		addpwdmem:
			mov		[iCpyCnt],0						; reset number of field counter
			
			;*====	Adjust remaining memory	=====
			sub		eax,edx							; subtract string's length
			mov		[iRemMem],eax					; update remaining string length
			
			;*====	Setup new word index	=====
			mov		eax,[iWdCount]					; get current count
			mov		[inWd],eax						; make it new index
			
			mov		ebx,[pnIndx]					; pointer to the new record index register
			mov		[ebx],eax						; save the new record index

			;*====	Save new word pointer	=====
			mov		edi,[pNxtWd]					; get pointer to next free location
			mov		[pnWd],edi						; save this address

			;*====	Copy the field data (string) into the word database	====
			mov		esi,[pStrWd]					; ESI -> start pointer to data field
			mov		ebx,[psDB]						; EBX -> Password database
			mov		eax,[inWd]						; EAX = new record index

			.if [bWdType] == 2
				;*====	Process for password database	====
				mov		ecx,[pCount]				; ECX -> pointer to the record counter
				mov		edx,[ecx]					; 
				shl		edx,3						; X 8 (form dword x 2)
				mov		ecx,edx						; copy 
				shl		edx,1						; x 4
				add		edx,eax						; X 6
				add		ebx,edx						; EBX -> start of password record
				inc		dword ptr [ecx]				; increment count
				
				.while [iCpyCnt] < 6
					push	ebx						; save the registers
					push	eax			
					call	get_strlen				; get string length
					mov		[edi],cl				; save string length
					inc		edi						; skip string length byte
					
					call	copy_null				; copy from field into destination
					inc		esi						; skip the null character
					
					;*====	Write the record indexes into the password database
					pop		eax						;
					pop		ebx						; EBX -> 
					mov		[ebx],eax				; write the index
					inc		eax						;  & then increment to next record
					
					inc		[iCpyCnt]				; increment count
				.endw
				
;				;*====	Copy the indexes into the source database	====
;				mov		edi,[psDB]					; EDI -> password database
;				mov		ebx,[pCount]				; pointer to the record counter
;				shl		ebx,3						; X 8
;				mov		eax,ebx						; copy 
;				shl		ebx,1						; x 4
;				add		ebx,eax						; X 6
;				add		edi,ebx						; EDI -> start of record
;				mov		eax,[inWd]					; start word index
;				
				mov		ecx,5						; 5 fields
			.else
				mov		ecx,[pCount]				; pointer to the record counter
				mov		edx,[ecx]					; 
				shl		edx,2						; x 4
				add		ebx,edx						; EDI -> start of record
				mov		[ebx],eax					; write group name record number
				inc		dword ptr [ecx]				; increment count
				
				dec		cl							; remove string's length byte count
				mov		[edi],cl					; save the string length first
				inc		edi							; skip the string's length
			
				call	copy_null					; copy the word into word database
				
				;*====	Copy the indexes into the source database	====
;				mov		edi,[psDB]					; EDI -> password database
;				mov		ebx,[pCount]				; pointer to the record counter
;				shl		ebx,2						; x 4
;				add		edi,ebx						; EDI -> start of record
;				mov		eax,[inWd]					; start word index
				
				mov		ecx,1						; 1 field
			.endif

			;*====	Write the indexes as field(s) into database	=====
			add		[iWdCount],ecx					; add new word count to its total
			
;			.while ecx > 0
;				mov		[edi],eax					; save index
;				inc		eax
;				dec		ecx
;			.endw
;				
;			mov		[iWdCount],eax					; save new word counter
			
			;*====	Increment the record counter	====			
;			mov		ebx,[pCount]					; pointer to the record counter
;			inc		dword ptr [ebx]					; increment the record counter
			
			;*====	Update the calling routine's indexes	====
			
			mov		eax,1							; return inserted status
			ret
			
Add_pword	endp

;*==================================================================================*
;*			Routine to delete password information from database					*
;*----------------------------------------------------------------------------------*
;*			Input:	EAX = Password index record to be deleted						*
;*																					*
;*			Output:	EAX = 0 Not deleted												*
;*						= 1 deleted													*
;*==================================================================================*
del_pwrec	proc
			cmp		eax,[iPWCount]					; valid index number
			ja		delpwerr						; invalid index number
			
			;*====	Proceed to unmark groupname field index	===========
			mov		edi,offset PWDB					; EDI -> point to start of database
			shl		eax,3							; X 8
			mov		ebx,eax							; copy 
			shl		eax,1							; x 4
			add		eax,ebx							; X 6
			add		edi,eax							; EDI -> start of record
			
			mov		eax,1							; return deleted error
			ret
			
		delpwerr:
			sub		eax,eax							; return not deleted status
			ret
del_pwrec	endp


;*==================================================================================*
;*			Routine to delete group name or password information into database		*
;*----------------------------------------------------------------------------------*
;*			Input variables															*
;*			---------------															*
;*			pStrWd				pointer to start of data to be inserted into DB		*
;*			pCount				pointer to record count								*
;*			pcIndx				pointer to current index							*
;*			pnIndx				pointer to new index								*
;*			psIndx				pointer to search index								*
;*			pcDB				pointer to start of database						*
;*			psDB				pointer to source database (group/password)			*
;*			iCpyCnt				number of string to copy							*
;*																					*
;*			Output:	EAX = 0 Not deleted												*
;*						= 1 deleted													*
;*						= -1 Indexes found in other records							*
;*==================================================================================*
del_pword	proc
;			push	ebp						; save EBP

;			.if [bWdType] == 2
;				;*====	Process to delete password	====
;				call	chk_pwindx			; check if this record has indexes in other record(s)
;			.else
;				sub		edx,edx				; default to not indexed
;			.endif
;			
;			;*====	Check if the record indexes are
;			.if edx == 0
;				mov		ebp,eax					; save index
;				inc		ebp						; increment index
;			
;			
;				mov		esi,[pCount]			; ESI -> record counter
;				cmp		ebp,[esi]				; at end of database?
;			.endif
			
;*			Input:	EAX = Index number, number is one base							*
;*					EDI -> Word database		 									*
;			mov		
;			pop		ebp						; restore EBP
			ret
del_pword	endp

;*==================================================================================*
;*			Routine to get the save a data											*
;*----------------------------------------------------------------------------------*
;*			This routine will save the database 							  		*
;*																					*
;*			Input:  EBX -> File path												*
;*					EDI -> Database 												*
;*					ECX = Bytes to save												*
;*																					*
;*			Output:	EAX = 	0 OK													*
;*							-1 Error												*
;*==================================================================================*
save_File		proc
			push	edi							; save registers
			push	ecx							;
										
			invoke	CreateFile, ebx, GENERIC_WRITE, FILE_SHARE_READ, 0, \
					CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0

			cmp		eax,INVALID_HANDLE_VALUE 	; does the file exits?
			je		savedberr			 		;
			
			mov		[hFile],eax					; save file handle
			
			pop		ecx							; restore byte to save
			pop		edi							; pointer to database
			
			push	ecx							; save size
			 
			;*----	WriteFile, hFile, Buffer, Bytes to save, bytes saved	----
			invoke	WriteFile, eax, edi, ecx, offset i_byteread, 00

			pop		ecx							; ECX = bytes to save
			cmp		ecx,[i_byteread]			; same number of bytes saved
			jne		savedb_byterr				; number of bytes saved error
			
			cmp		eax,FALSE					; save OK?
			je		writeError					; No, error
			
			invoke	CloseHandle, hFile			; close file
			sub		eax,eax						; return OK status
		saveEx:
			pop		edi
			ret

			;*====	Error opening file for write	====
		savedberr:
			mov		eax,-1						;return error
		saveErrCode:
			push	eax							; save return code
			
			invoke	CloseHandle, hFile			; close file
       	    invoke	GetLastError				;
       	    mov		[FileError],eax
       	    
       	    pop		eax							; get return code back
			jmp		saveEx

			;*====	Error in byte written count	====
		savedb_byterr:
			mov		eax,-2						;return error
			jmp		saveErrCode
		
			;*====	Unable to save file	====
		writeError:
			mov		eax,-3						;return error
			jmp		saveErrCode
		
save_File 	endp

;*==================================================================================*
;*			Routine to get the load a database										*
;*----------------------------------------------------------------------------------*
;*			This routine will load the database 							  		*
;*																					*
;*			Input:  EBX -> File path												*
;*					EDI -> Database 												*
;*					ECX = Bytes to load												*
;*																					*
;*			Output:	EAX = 	0 OK													*
;*							-1 Error												*
;*==================================================================================*
load_File		proc
			push	edi
			push	ecx
	
			;*====	Open file for read operation	====
			invoke	CreateFile, ebx, GENERIC_READ, FILE_SHARE_READ, 0, \
					OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0

			cmp		eax,INVALID_HANDLE_VALUE ; does the file exits?
			je		loadDBerr				 ;
					
			;*----	Valid file and is to be read	-----
			mov		[hFile],eax				; save file handle

			pop		ecx						; restore read byte count
			pop		edi						;  & destination buffer
			
			push	ecx						; save count again for use later
			
			;*----	Read the whole file into file buffer	----
			;*----	ReadFile, hFile, lpBuffer, NoofBytesToRead, NoofBytesRead, lpOverlapped
			invoke	ReadFile, hFile, edi, ecx, offset i_byteread, 00
				
			cmp		eax,FALSE				; read operation error
			je		loadDBerr				; process error

			invoke	CloseHandle, hFile		; close file
			sub		eax,eax					; done

			;*====	Prepare to exit - restore stack	====
	loadEx:
			pop		ecx
			pop		edi
			ret

	loadDBerr:
			invoke	CloseHandle, hFile		; close file
			mov		eax,-1					; error loading file
       	    invoke	GetLastError			;
			jmp		loadEx
load_File 	endp

;*==================================================================================*
;*			Routine to get the load/save the database								*
;*----------------------------------------------------------------------------------*
;*			Input:	AX 	= 1  Load database											*
;*						= 2	 Save database											*
;*==================================================================================*
ldsv_DB		proc
;			push	ax							; save command code
;			
;			mov		esi,offset FilePath			; file path
;			mov		edi,offset FPath			; working path
;			mov		ebx,offset DB_FileName		; database filename
;			call	combine_path_file			; concentanate file name, path into working path
;
;			mov		ebx,edi						; EDI -> database path name
;			mov		edi,offset WData			; EDI -> database
;			mov		eax,[WRecords]				; get record count
;			mov		ecx,WRecSize				; record count
;			mul		cx							; 
;			mov		ecx,eax						; byte count to save 
;
;			pop		ax							; restore command code
;			.if ax == LOADDB
;				call	load_File				; load database
;			.else
;				call	save_File				; save database
;			.endif
		
			ret

ldsv_DB 	endp

;*==================================================================================*
;*			Routine to get the load system record									*
;*----------------------------------------------------------------------------------*
;*			Input:	AX 	= 1  Load system record										*
;*						= 2	 Save system record										*
;*==================================================================================*
ldsv_Sys	proc
			push	ax
;			mov		edi,offset FPath			; working path
;			sub		eax,eax						;
;			mov		ecx,64
;			rep		stosd						; clear the destination buffer
;			
;			mov		esi,offset FilePath			; file path
;			mov		edi,offset FPath			; working path
;			mov		ebx,offset Sys_FileName		; system filename
;			call	combine_path_file			; concentanate file name, path into working path
;
;			mov		ebx,edi			
;			mov		edi,offset WRecords			; EDI -> database
;			mov		ecx,320						; record count
;
;			pop		ax
;			.if ax == LOADSYS
;				call	load_File				; load system record
;			.else
;				call	save_File				; save system record				
;			.endif
		
			ret

ldsv_Sys 	endp

;*==================================================================================*
;*			Routine to get the load system record									*
;*==================================================================================*
save_Sys	proc
;			mov		edi,offset FPath			; working path
;			sub		eax,eax						;
;			mov		ecx,64
;			rep		stosd						; clear the destination buffer
;			
;			mov		esi,offset FilePath			; file path
;			mov		edi,offset FPath			; working path
;			mov		ebx,offset Sys_FileName		; system filename
;			call	combine_path_file			; concentanate file name, path into working path
;
;			mov		ebx,edi			
;			mov		edi,offset WRecords			; EDI -> database
;			mov		ecx,320						; record count

			call	save_File						; save the database
			
			ret

save_Sys 	endp

